@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2002-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -37,11 +37,14 @@
 #define INPUT    MakeArray<String>
 
 /*! The lookup information for the agendas. */
+namespace global_data {
 Array<AgRecord> agenda_data;
-
+}
 
 void define_agenda_data()
 {
+  using global_data::agenda_data;
+
   // Initialize to zero, just in case:
   agenda_data.resize(0);
 
@@ -53,31 +56,70 @@ void define_agenda_data()
 
   agenda_data.push_back
     (AgRecord
-     ( NAME( "abs_scalar_gas_agenda" ),
+     ( NAME( "propmat_clearsky_agenda" ),
        DESCRIPTION
        (
-        "Calculation of scalar gas absorption.\n"
-        "\n"
-        "This agenda calculates absorption coefficients for all gas species\n"
-        "as a function of the given atmospheric state for one point in the\n"
-        "atmosphere. The result is returned in *abs_scalar_gas*, the\n"
-        "atmospheric state has to be specified by *rte_pressure*,\n"
-        "*rte_temperature*, and *rte_vmr_list*.\n"
-        "\n"
-        "A mandatory input parameter is f_index, which is used as follows:\n"
-        "\n"
-        "1. f_index < 0 : Return absorption for all frequencies (in f_grid).\n"
-        "\n"
-        "2. f_index >= 0 : Return absorption for the frequency indicated by\n"
-        "   f_index. \n"
+        "This agenda calculates the absorption coefficient matrix for all\n"
+        "absorption species as a function of the given atmospheric state for\n"
+        "one point in the atmosphere. The result is returned in\n"
+        "*propmat_clearsky*. The atmospheric state has to be specified by\n"
+        "*rtp_pressure*, *rtp_temperature*, *rtp_mag*, and *rtp_vmr*.\n"
         "\n"
         "The methods inside this agenda may require a lot of additional\n"
-        "input variables, such as *f_grid*, *abs_species*, etc.\n"
+        "input variables, such as *abs_species*, etc.\n"
+        "\n"
+        "The include file 'agendas.arts' predefines some possible agendas\n"
+        "that can be used here.\n"
         ),
-       OUTPUT( "abs_scalar_gas" ),
-       INPUT(  "f_index", "rte_doppler", "rte_pressure", "rte_temperature", 
-               "rte_vmr_list" )));
+       OUTPUT( "propmat_clearsky" ),
+       INPUT(  "f_grid", "rtp_mag", "rtp_los", "rtp_pressure", 
+               "rtp_temperature", "rtp_vmr" )));
   
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "abs_xsec_agenda" ),
+      DESCRIPTION
+      (
+       "Calculate scalar gas absorption cross sections.\n"
+       "\n"
+       "Basically, this agenda calculates absorption for all the tags defined\n"
+       "in abs_species. It is used both in the calculation of an absorption\n"
+       "lookup table, and in on-the-fly calculations. Typical effects to\n"
+       "include here are:\n"
+       "\n"
+       "Explicit line-by-line calculation (*abs_xsec_per_speciesAddLines*),\n"
+       "\n"
+       "Continua and complete absorption models (*abs_xsec_per_speciesAddConts*), and\n"
+       "\n"
+       "HITRAN style CIA continua (*abs_xsec_per_speciesAddCIA*)\n"
+       "\n"
+       "The only kind of absorption tag not handled here are Zeeman tags\n"
+       "and free electron density tags, because they need additional input\n"
+       "and because they return an absorption matrix, rather than a scalar.\n"
+       "\n"
+       "The include file 'agendas.arts' predefines a number of agendas that\n"
+       "should be useful for most users.\n"
+       ),
+      OUTPUT( "abs_xsec_per_species" ),
+      INPUT(  "abs_species", "abs_species_active",
+              "f_grid", "abs_p", "abs_t", "abs_vmrs" )));
+
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "blackbody_radiation_agenda" ),
+       DESCRIPTION
+       (
+        "Returns *blackbody_radiation*.\n"
+        "\n"
+        "The task of this agenda is to return *blackbody_radiation* for the\n"
+        "given temperature (*rtp_temperature*).\n"
+        "\n"
+        "The include file 'agendas.arts' predefines an agenda that can be\n"
+        "used here.\n"
+        ),
+       OUTPUT( "blackbody_radiation" ),
+       INPUT( "rtp_temperature", "f_grid" )));
+ 
  agenda_data.push_back
     (AgRecord
      ( NAME( "doit_conv_test_agenda" ),
@@ -102,6 +144,31 @@ void define_agenda_data()
        INPUT(  "doit_conv_flag", "doit_iteration_counter",
                "doit_i_field", "doit_i_field_old" )));
 
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "doit_mono_agenda" ),
+       DESCRIPTION
+       (
+       "Performs monochromatic DOIT calculation."
+       "\n"
+       "This agenda includes for example the following methods:\n"
+       "   1. *DoitScatteringDataPrepare* \n"
+       "   2. *doit_i_fieldSetClearsky* \n"
+       "   3. *doit_i_fieldIterate*\n"
+       "   4. *DoitCloudboxFieldPut*\n"
+       "\n"
+       "The result of the agenda is the radiation field inside the \n"
+       "cloudbox and on the cloudbox boundary, which can be used \n"
+       "as radiative background for a clearsky radiative transfer \n"
+       "calculation. \n"
+       "\n"
+       "See the ArtsWiki page UsingArtsDoit and the online documentation\n"
+       "for more information about the methods.\n"
+        ),
+       OUTPUT( "doit_i_field", "scat_i_p", "scat_i_lat", "scat_i_lon", 
+               "doit_i_field1D_spectrum"),
+       INPUT("doit_i_field", "f_grid", "f_index", "scat_i_p", "scat_i_lat", "scat_i_lon")));
+            
  agenda_data.push_back
     (AgRecord
      ( NAME( "doit_scat_field_agenda" ),
@@ -128,31 +195,6 @@ void define_agenda_data()
         OUTPUT( "doit_scat_field" ),
         INPUT(  "doit_scat_field", "doit_i_field")));
 
-  agenda_data.push_back
-    (AgRecord
-     ( NAME( "doit_mono_agenda" ),
-       DESCRIPTION
-       (
-       "Performs monochromatic DOIT calculation."
-       "\n"
-       "This agenda includes for example the following methods:\n"
-       "   1. *DoitScatteringDataPrepare* \n"
-       "   2. *doit_i_fieldSetClearsky* \n"
-       "   3. *doit_i_fieldIterate*\n"
-       "   4. *DoitCloudboxFieldPut*\n"
-       "\n"
-       "The result of the agenda is the radiation field inside the \n"
-       "cloudbox and on the cloudbox boundary, which can be used \n"
-       "as radiative background for a clearsky radiative transfer \n"
-       "calculation. \n"
-       "\n"
-       "See the ArtsWiki page UsingArtsDoit and the online documentation\n"
-       "for more information about the methods.\n"
-        ),
-       OUTPUT( "doit_i_field", "scat_i_p", "scat_i_lat", "scat_i_lon", 
-               "doit_i_field1D_spectrum"),
-       INPUT("f_index", "scat_i_p", "scat_i_lat", "scat_i_lon")));
-            
  agenda_data.push_back
     (AgRecord
      ( NAME( "doit_rte_agenda" ),
@@ -185,21 +227,6 @@ void define_agenda_data()
         OUTPUT( "doit_i_field" ),
        INPUT(   "doit_i_field", "doit_scat_field" )));
  
-  agenda_data.push_back
-    (AgRecord
-     ( NAME( "emission_agenda" ),
-       DESCRIPTION
-       (
-        "Thermal emission source term.\n"
-        "\n"
-        "This agenda shall return the emission at one position along\n"
-        "the propagation path. The source term equals the Planck function as\n"
-        "long as thermodynamic equilibrium (LTE) can be assumed, while for\n"
-        "non-LTE conditions much more complex calculations are required.\n"
-        ),
-       OUTPUT( "emission" ),
-       INPUT( "rte_temperature" )));
- 
   agenda_data.push_back
     (AgRecord
      ( NAME( "forloop_agenda" ),
@@ -212,6 +239,29 @@ void define_agenda_data()
         ),
        OUTPUT(),
        INPUT( "forloop_index" )));
+ 
+  /*
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "fos_y_agenda" ),
+       DESCRIPTION
+       (
+        "Calculation of incoming radiation field for FOS method.\n"
+        "\n"
+        "The direct task of the agenda is to determine the incoming radiation\n"
+        "field, to evaluate of the scattering integral, for each angle in\n"
+        "*fos_angle*. The data are packed into *fos_y*.\n"
+        "\n"
+        "The underlying purpose of this agenda is to allow different\n"
+        "shortcuts for estimating the incoming radiation field. For example,\n"
+        "calculations can be performed for a few directions and then an\n"
+        "interpolation is performed to obtain the intensity for other\n"
+        "directions. The data could also be taken from a pre-calculated\n"
+        "database.\n"
+        ),
+       OUTPUT( "fos_y" ),
+       INPUT( "rte_pos", "fos_angles", "fos_n", "fos_i" )));
+  */
 
 //   agenda_data.push_back
 //     (AgRecord
@@ -240,62 +290,62 @@ void define_agenda_data()
 
   agenda_data.push_back
     (AgRecord
-     ( NAME( "iy_clearsky_agenda" ),
+     ( NAME( "g0_agenda" ),
        DESCRIPTION
        (
-        "Calculation of a single monochromatic pencil beam spectrum.\n"
+        "Calculation of the gravity at zero altitude.\n"
         "\n"
-        "The task of the agenda is to calculate the monochromatic pencil beam\n"
-        "spectrum for the position specified by *rte_pos* and the viewing\n"
-        "direction specified by *rte_los*. This includes cases when the\n"
-        "propagation path intersects with the surface or the cloudbox.\n"
+        "Returns *g0* for given geographical position.\n"
         ),
-       OUTPUT( "iy", "iy_error", "iy_error_type", "iy_aux", "diy_dx" ),
-       INPUT( "iy_error", "iy_error_type", "iy_aux", "diy_dx",
-              "iy_agenda_call1", "iy_transmission",
-              "rte_pos", "rte_los", "cloudbox_on", "jacobian_do", 
-              "p_grid", "lat_grid", "lon_grid", "t_field", "z_field", 
-              "vmr_field" )));
-
-  agenda_data.push_back
-    (AgRecord
-     ( NAME( "iy_clearsky_basic_agenda" ),
-       DESCRIPTION
-       (
-        "As *iy_clearsky_agenda*, but lacks all support for jacobian and\n"
-        "auxiliary variables.\n"
-        "\n"
-        "This agenda is used by scattering methods without support for the\n"
-        "jacobian and auxilary parts, in order to calculate unscattered\n"
-        "radiation for the speciefied position and line-of-sight. As this\n"
-        "agenda has much fewer input variables, this simplifies the interface\n"
-        "and maintaince of those scattering methods.\n"
-        ),
-       OUTPUT( "iy" ),
-       INPUT( "rte_pos", "rte_los", "cloudbox_on" )));
+       OUTPUT( "g0" ),
+       INPUT( "lat", "lon" )));
 
   agenda_data.push_back
     (AgRecord
      ( NAME( "iy_cloudbox_agenda" ),
        DESCRIPTION
        (
-        "Intensity at cloudbox boundary or interior.\n"
+        "Intensity at boundary or interior of the cloudbox.\n"
         "\n"
         "The task of the agenda is to determine the intensity at some point\n"
-        "at the boundary of inside the cloudbox.  The actual calculations\n"
+        "at the boundary of or inside the cloudbox. The actual calculations\n"
         "inside the agenda differ depending on scattering solution method.\n"
         "If DOIT is used, an interpolating of the intensity field should be\n"
         "performed. Another option is to start backward Monte Carlo \n"
         "calculations from this point.\n"
         "\n"
-        "A function calling this agenda shall set *rte_pos*, *rte_los* and\n"
-        "*rte_gp_...* variables to the position and line-of-sight for which the\n"
-        "scattered radiation shall be determined.\n"
+        "A function calling this agenda shall set *rte_pos* and *rte_los* to\n"
+        "the position and line-of-sight for which the scattered radiation\n"
+        "shall be determined.\n"
+        "\n"
+        "The include-file 'agendas.arts' pre-defines some agendas that can\n"
+        "either be used directly, or serve as examples.\n"
         ),
-       OUTPUT( "iy", "iy_error", "iy_error_type", "iy_aux", "diy_dx" ),
-       INPUT( "iy_error", "iy_error_type", "iy_aux", "diy_dx", 
-              "iy_transmission",
-              "rte_pos", "rte_los", "rte_gp_p", "rte_gp_lat", "rte_gp_lon" )));
+       OUTPUT( "iy" ),
+       INPUT( "f_grid", "rtp_pos", "rtp_los" )));
+
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "iy_main_agenda" ),
+       DESCRIPTION
+       (
+        "Calculation of a single monochromatic pencil beam spectrum.\n"
+        "\n"
+        "The task of the agenda is to calculate the monochromatic pencil beam\n"
+        "spectrum for the position specified by *rte_pos* and the viewing\n"
+        "direction specified by *rte_los*.\n"
+        "\n"
+        "Methods for this agenda can either handle the complete calculation,\n"
+        "make use of e.g. *iy_cloudbox_agenda* or be restricted to special\n"
+        "cases. See the documentation for the different methods.\n"
+        "\n"
+        "The include-file 'agendas.arts' predefines some typical alternatives\n"
+        "that can be used directly, or adapted for specific applications.\n"
+        ),
+       OUTPUT( "iy", "iy_aux", "ppath", "diy_dx" ),
+       INPUT( "iy_agenda_call1", "iy_transmission", "iy_aux_vars",
+              "cloudbox_on", "jacobian_do", "t_field", "z_field", "vmr_field", 
+              "f_grid", "rte_pos", "rte_los", "rte_pos2" )));  
 
   agenda_data.push_back
     (AgRecord
@@ -307,13 +357,78 @@ void define_agenda_data()
         "Possible terms to include in this agenda include cosmic background\n"
         "radiation and solar radiation.\n"
         "\n"
-        "A function calling this agenda shall set *rte_pos* and *rte_los* to\n"
+        "A function calling this agenda shall set *rtp_pos* and *rtp_los* to\n"
         "the position and line-of-sight for which the entering radiation \n"
         "shall be determined. The position and line-of-sight must be known, \n"
         "for example, when radiation from the sun is considered.\n"
+        "\n"
+        "The include-file 'agendas.arts' predefines an agenda that can be\n"
+        "applied directly for most users.\n"
         ),
        OUTPUT( "iy" ),
-       INPUT( "rte_pos", "rte_los" )));
+       INPUT( "f_grid", "rtp_pos", "rtp_los" )));
+
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "iy_sub_agenda" ),
+       DESCRIPTION
+       (
+        "Sub-agenda to *iy_main_agenda*. \n"
+        "\n"
+        "The purpose of this agenda is to make it possible to introduce a\n"
+        "second layer inside *iy_main_agenda*. That is, *iy_main_agenda*\n"
+        "calls this agenda to obtain *iy* and associated variables. See\n"
+        "*iyLoopFrequencies* for example usage.\n"
+        "\n"
+        "This agenda has exactly the same in- and output as *iy_main_agenda*.\n"
+        ),
+       OUTPUT( "iy", "iy_aux", "ppath", "diy_dx" ),
+       INPUT( "iy_agenda_call1", "iy_transmission", "iy_aux_vars",
+              "cloudbox_on", "jacobian_do", "t_field", "z_field", "vmr_field", 
+              "f_grid", "rte_pos", "rte_los", "rte_pos2" )));  
+
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "iy_surface_agenda" ),
+       DESCRIPTION
+       (
+        "Upwelling radiation from the surface.\n"
+        "\n"
+        "The task of the agenda is to determine the upwelling intensity from\n"
+        "the surface, for given point and direction.\n"
+        "\n"
+        "The standard choice should be to make use of *surface_rtprop_agenda*\n"
+        "through the WSM *iySurfaceRtpropAgenda*.\n"
+        "\n"
+        "A function calling this agenda shall set *rtp_pos* and *rtp_los* to\n"
+        "the position and line-of-sight for which the upwelling radiation\n"
+        "shall be determined.\n"
+        "\n"
+        "See also the include-file 'agendas.arts' for a predefined agenda\n"
+        "suitable to be used in most applications.\n"
+        ),
+       OUTPUT( "iy", "diy_dx" ),
+       INPUT( "iy_transmission", "cloudbox_on", 
+              "jacobian_do", "t_field", "z_field", "vmr_field", "f_grid",
+              "iy_main_agenda", "rtp_pos", "rtp_los", "rte_pos2" )));
+
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "iy_transmitter_agenda" ),
+       DESCRIPTION
+       (
+        "Transmitted signal.\n"
+        "\n"
+        "This agenda describes the signal at the start of the propagation\n"
+        "path for calculations of transmission type. That is, the agenda\n"
+        "describes a transmitter, which either can be a natural source or\n"
+        "an artificial device.\n"
+        "\n"
+        "The include-file 'agendas.arts' defines an example agenda that\n"
+        "can be used for transmission calculations\n"
+        ),
+       OUTPUT( "iy" ),
+       INPUT( "f_grid", "rtp_pos", "rtp_los" )));
 
   agenda_data.push_back
     (AgRecord
@@ -331,24 +446,7 @@ void define_agenda_data()
         "by calling the the jacobianAdd set of methods.\n"
        ),
        OUTPUT( "jacobian" ),
-       INPUT( "jacobian", "imblock", "iyb", "yb" )));
-
-  agenda_data.push_back
-    (AgRecord
-     ( NAME( "jacobian_y_agenda" ),
-       DESCRIPTION
-       (
-        "Agenda providing recalculated *y* after some perturbation.\n"
-        "\n"
-        "The purpose of this agenda is to determine some jacobians through\n"
-        "perturbation calculations. Accordingly, the agenda shall return *y*\n"
-        "for the perturbed input (without doing any unnecessary operations,\n"
-        "for efficiency reasons). If unperturbed spectra (and analytical\n"
-        "jacobians) are calculated with *RteCalc*, the standard choice for\n"
-        "this agenda should be *RteCalcNoJacobians*.\n"
-       ),
-       OUTPUT( "y" ),
-       INPUT( "f_grid", "vmr_field", "t_field", "sensor_los" )));
+       INPUT( "jacobian", "mblock_index", "iyb", "yb" )));
 
   agenda_data.push_back
     (AgRecord
@@ -390,44 +488,6 @@ void define_agenda_data()
              "p_grid", "sensor_los", "cloudbox_on", "cloudbox_limits",
              "z_surface")));
 
-  agenda_data.push_back
-    (AgRecord
-     ( NAME( "opt_prop_gas_agenda" ),
-       DESCRIPTION
-       (
-        "Calculate the optical properties (absorption vector and extinction.\n"
-        "matrix) of gaseous species at a given grid point.\n"
-        "\n"
-        "This agenda, for example, can be defined in the following manner:\n"
-        "\n"
-        "*ext_matAddGas* : This method calculates the extinction \n"
-        "                  matrix for the gaseous species and adds it to \n"
-        "                  the workspace variable *ext_mat*.\n"
-        "*abs_vecAddGas* : This method calculates the absorption\n"
-        "                  vector for the gaseous species and adds it to\n"
-        "                  the workspace variables abs_vec.\n"     
-        "If the Zeeman effect should be included the following methods have \n"
-        "to be added: \n"
-        "*ext_matAddZeeman* \n"
-        "*abs_vecAddZeeman* \n"
-        " \n"
-        "Note that the initialization of *abs_vec* is not done inside the\n"
-        "agenda, so *abs_vec* has to be initialize before executing the \n"
-        "agenda.\n"
-        "\n"
-        "Output :\n"
-        "   ext_mat     : Extinction matrix.\n"
-        "   abs_vec     : Absorption vector.\n"
-        "\n"
-        "Input:\n"
-        "   ext_mat     : Extinction matrix.\n"
-        "   abs_vec     : Absorption vector. \n"
-        "   abs_scalar_gas : Scalar gas absorption. \n"
-        ),
-       OUTPUT( "ext_mat", "abs_vec" ),
-       INPUT( "ext_mat", "abs_vec", "f_index", "abs_scalar_gas" )));
-
-
  agenda_data.push_back
     (AgRecord
      ( NAME( "opt_prop_part_agenda" ),
@@ -482,8 +542,32 @@ void define_agenda_data()
         ),
        OUTPUT( "pha_mat_spt"),
        INPUT( "pha_mat_spt", "scat_za_index", "scat_lat_index", "scat_lon_index",
-              "scat_p_index", "scat_aa_index", "rte_temperature")));
+              "scat_p_index", "scat_aa_index", "rtp_temperature")));
        
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "ppath_agenda" ),
+       DESCRIPTION
+       (
+        "Calculation of complete propagation paths.\n"
+        "\n"
+        "In contrast to *ppath_step_agenda* that controls the ray tracing\n"
+        "inside each grid box, this agenda determines how complete paths are\n"
+        "determined. The standard choice is to do this in a step-by-step\n"
+        "manner using *ppath_step_agenda*, with this agenda set to call\n" 
+        "*ppathStepByStep*.\n" 
+        "\n"
+        "The WSV *rte_los* is both input and output as in some cases it is\n"
+        "determined as part of the propagation path calculations (such as for"
+        "for radio link calculations).\n"
+        "\n"
+        "The include file 'agendas.arts' predefines some agendas that can\n"
+        "either be used directly, or serve as inspiration.\n"
+        ),
+       OUTPUT( "ppath" ),
+       INPUT( "ppath_lraytrace", "rte_pos", "rte_los", "rte_pos2",
+              "cloudbox_on", "ppath_inside_cloudbox_do", "t_field", "z_field", 
+              "vmr_field", "f_grid" )));
 
   agenda_data.push_back
     (AgRecord
@@ -517,14 +601,8 @@ void define_agenda_data()
         "This is either the position of the sensor (true or hypothetical), \n"
         "or some point at the top of the atmosphere (determined by\n" 
         "geometrical calculations starting at the sensor). This \n" 
-        "initialisation is not handled by *ppath_step_agenda*. All fields of\n"
-        "*ppath_step* are set by *ppath_step_agenda*. If the sensor is above\n"
-        "the model atmosphere the field *constant* can be initiated by the \n" 
-        "calling method. Otherwise the field shall be set to negative and it\n"
-        "is set to the correct value by *ppath_step* at the first call. This\n"
-        "procedure is needed as the path constant changes if refraction is \n"
-        "considered, or not, when the sensor is placed inside the\n" 
-        "atmosphere.\n"
+        "initialisation is not handled by *ppath_step_agenda* (but by \n"
+        "the internal function ppath_start_stepping). \n"
         "\n"
         "The *ppath_step_agenda* put in points along the propagation path \n"
         "at all crossings with the grids, tangent points and points of \n"
@@ -532,23 +610,30 @@ void define_agenda_data()
         "additional points to fulfil some criterion, such as a maximum \n"
         "distance along the path between the points. Accordingly, the \n"
         "number of new points of each step can exceed one.\n"
+        "\n"
+        "The include file 'agendas.arts' defines some agendas that can be\n"
+        "used here."
         ),
        OUTPUT( "ppath_step" ),
-       INPUT( "ppath_step", "atmosphere_dim", "p_grid", "lat_grid",
-              "lon_grid", "z_field", "r_geoid", "z_surface" )));
+       INPUT( "ppath_step", "ppath_lraytrace", "t_field", "z_field", 
+              "vmr_field", "f_grid" )));
 
   agenda_data.push_back
     (AgRecord
-     ( NAME( "refr_index_agenda" ),
+     ( NAME( "refr_index_air_agenda" ),
        DESCRIPTION
        (
         "Calculation of the refractive index of air.\n"
         "\n"
         "This agenda should calculate the summed refractive index for all\n"
-        "relevant atmospheric constituients.\n"
+        "relevant atmospheric constituents, with respect to both phase and\n"
+        "group velocity.\n"
+        "\n"
+        "The include file 'agendas.arts' predefines several agendas that\n"
+        "may either be used directly, or serve as inspiration.\n"
         ),
-       OUTPUT( "refr_index" ),
-       INPUT(  "rte_pressure", "rte_temperature", "rte_vmr_list" )));
+       OUTPUT( "refr_index_air", "refr_index_air_group" ),
+       INPUT(  "rtp_pressure", "rtp_temperature", "rtp_vmr", "f_grid" )));
 
   agenda_data.push_back
     (AgRecord
@@ -558,11 +643,11 @@ void define_agenda_data()
         "The sensor response data for present measurement block.\n"
         "\n"
         "This agenda shall provide *sensor_response* and associated variables\n"
-        "for the present measurement block (*imblock*).\n"
+        "for the present measurement block (*mblock_index*).\n"
         ),
        OUTPUT( "sensor_response", "sensor_response_f", "sensor_response_pol",
                "sensor_response_za", "sensor_response_aa" ),
-       INPUT(  "imblock" )));
+       INPUT(  "mblock_index" )));
 
  agenda_data.push_back
     (AgRecord
@@ -581,22 +666,36 @@ void define_agenda_data()
        OUTPUT( "ext_mat_spt", "abs_vec_spt"),
        INPUT(  "ext_mat_spt", "abs_vec_spt",
                "scat_p_index", "scat_lat_index", "scat_lon_index",
-               "rte_temperature", "scat_za_index", "scat_aa_index"
+               "rtp_temperature", "scat_za_index", "scat_aa_index"
                )));
 
   agenda_data.push_back
     (AgRecord
-     ( NAME( "surface_prop_agenda" ),
+     ( NAME( "surface_rtprop_agenda" ),
        DESCRIPTION
        (
-        "Surface radiative properties. \n"
+        "Provides radiative properties of the surface. \n"
         "\n"
-        "See the user guide for closer definitions of the variables \n"
-        "that describe the surface properties. These variables are:\n"
-        "   *surface_emission*, *surface_los* and *surface_rmatrix* \n"
+        "Provides surface emission and surface reflection coefficient matrix\n"
+        "(see user guide for closer definitions of the respective variables\n"
+        "*surface_emission*, *surface_los*, and *surface_rmatrix*) according\n"
+        "to the characteristics of the surface specified by the methods called\n"
+        "within the agenda. Typical meyhods include *surfaceBlackbody*,\n"
+        "*surfaceFlatScalarReflectivity*, *surfaceFlatReflectivity*,\n"
+        "*surfaceFlatRefractiveIndex*, and *surfaceLambertianSimple*.\n"
         ),
        OUTPUT( "surface_emission", "surface_los", "surface_rmatrix" ),
-       INPUT( "rte_pos", "rte_los", "rte_gp_p", "rte_gp_lat", "rte_gp_lon" )));
+       INPUT( "f_grid", "rtp_pos", "rtp_los" )));
+
+  agenda_data.push_back
+    (AgRecord
+     ( NAME( "test_agenda" ),
+       DESCRIPTION
+       (
+       "Dummy agenda for testing purposes.\n"
+       ),
+      OUTPUT(),
+      INPUT()));
 
  agenda_data.push_back
     (AgRecord
@@ -608,47 +707,7 @@ void define_agenda_data()
         "Must produce a new spectrum vector (*y*) and Jacobi matrix (*jacobian*).\n"
         "See further *ybatchCalc*.\n"
         ),
-       OUTPUT( "y", "jacobian" ),
+       OUTPUT( "y", "y_aux", "jacobian" ),
        INPUT( "ybatch_index" )));
 
-  agenda_data.push_back
-  (AgRecord
-   ( NAME( "test_agenda" ),
-     DESCRIPTION
-     (
-      "Dummy agenda for testing purposes.\n"
-     ),
-     OUTPUT(),
-     INPUT()));
-  
-
-//  agenda_data.push_back
-//     (AgRecord
-//      ( NAME( "zeeman_prop_agenda" ),
-//        DESCRIPTION
-//        (
-//         "Calculates extinction matrix and absorption vector due to the \n"
-//         "Zeeman effect. \n"
-//         "\n"
-//         "The agenda calculates the total extinction matrix and absorption \n"
-// 	"vector of O2 only (currently) due to the Zeeman effect induced by the \n"
-// 	"geomagnetic field. The polarization pattern, resulting from the effect, \n"
-// 	"produces additional contribution to these quantities in the unpolarized \n"
-// 	"case. This means that the user should take care not to execute both the \n"
-// 	"polarized (Zeeman) and unpolarized  calculation for the O2 lines \n"
-// 	"affected by the Zeeman effect, otherwise the unpolarized part gets \n"
-// 	"wrongly doubled. \n"
-//         "\n"
-//         "Output:    \n"
-//         "   ext_mat_zeeman: Zeeman extinction matrix  \n"
-//         "   abs_vec_zeeman: Zeeman absorption vector  \n"
-//         "\n"
-//         "Input:    \n"
-//         "  geomag_los: Magnetic field along LOS plus angle  \n"
-//         "\n"
-//           ),
-//        OUTPUT(  ),
-//        INPUT(  )));
-
-
 }
