@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2001-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -60,6 +60,8 @@ class Tensor7;
 /** An array of vectors. */
 typedef Array<Vector> ArrayOfVector;
 
+typedef Array<ArrayOfVector> ArrayOfArrayOfVector;
+
 /** An array of matrices. */
 typedef Array<Matrix> ArrayOfMatrix;
 
@@ -76,6 +78,8 @@ typedef Array<ArrayOfTensor3> ArrayOfArrayOfTensor3;
 /** An array of Tensor4. */
 typedef Array<Tensor4> ArrayOfTensor4;
 
+typedef Array<ArrayOfTensor4> ArrayOfArrayOfTensor4;
+
 /** An array of Tensor5. */
 typedef Array<Tensor5> ArrayOfTensor5;
 
@@ -99,14 +103,14 @@ typedef Array<Tensor7> ArrayOfTensor7;
     constructors are provided by the derived class MakeArray.
 */
 template<class base>
-class Array : public vector<base>
+class Array : public std::vector<base>
 {
 public:
   // Constructors:
-  Array()                     : vector<base>()  { /* Nothing to do here. */ }
-  explicit Array(Index n)     : vector<base>(n) { /* Nothing to do here. */ }
+  Array()                     : std::vector<base>()  { /* Nothing to do here. */ }
+  explicit Array(Index n)     : std::vector<base>(n) { /* Nothing to do here. */ }
   Array(Index n, const base& fillvalue);
-  Array(const Array<base>& A) : vector<base>(A) { /* Nothing to do here. */ }
+  Array(const Array<base>& A) : std::vector<base>(A) { /* Nothing to do here. */ }
 
   // Assignment operators:
   Array& operator=(base x);
@@ -127,7 +131,7 @@ public:
 /** Constructor filling with constant value. */
 template<class base>
 inline Array<base>::Array(Index n, const base& fillvalue) :
-  vector<base>(n)
+  std::vector<base>(n)
 {
   // Use fill to fill.
   fill(this->begin(),this->end(),fillvalue);
@@ -183,17 +187,17 @@ inline const base& Array<base>::operator[](Index n) const
 {
   assert(0<=n);
   assert(n<nelem());
-  return vector<base>::operator[](n);
+  return std::vector<base>::operator[](n);
 }
 
-/** Non-constant index operator. We redifine this here so that we can
+/** Non-constant index operator. We redefine this here so that we can
     have range checking by assert. */
 template<class base>
 inline base& Array<base>::operator[](Index n)
 {
   assert(0<=n);
   assert(n<nelem());
-  return vector<base>::operator[](n);
+  return std::vector<base>::operator[](n);
 }
 
 
@@ -201,20 +205,20 @@ inline base& Array<base>::operator[](Index n)
 
 /** Output operator. */
 template<class base>
-inline ostream& operator<<(ostream& os, const Array<base>& v)
+inline std::ostream& operator<<(std::ostream& os, const Array<base>& v)
 {
   typename Array<base>::const_iterator         i = v.begin();
   const typename Array<base>::const_iterator end = v.end();
 
   if ( i!=end )
     {
-      os << setw(3) << *i;
+      os << std::setw(3) << *i;
       ++i;
     }
 
   for ( ; i!=end; ++i )
     {
-      os << " " << setw(3) << *i;
+      os << " " << std::setw(3) << *i;
     }
 
   return os;
