@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2002-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,6 +25,9 @@
 #include "matpackV.h"
 #include "exceptions.h"
 
+using std::runtime_error;
+
+
 // Functions for ConstTensor5View:
 // ------------------------------
 
@@ -756,14 +759,14 @@ is not 1 because the caller expects to get a C array with continuous data.
 */
 Numeric *Tensor5View::get_c_array()
 {
-  if (msr.mstart != 0 || msr.mstride != mbr.mextent * mpr.mextent * mrr.mextent * mcr.mextent
-      || mbr.mstart != 0 || mbr.mstride != mpr.mextent * mrr.mextent * mcr.mextent
-      || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
-      || mrr.mstart != 0 || mrr.mstride != mcr.mextent
-      || mcr.mstart != 0 || mcr.mstride != 1)
-    throw runtime_error("A Tensor5View can only be converted to a plain C-array if mbr.mstart == 0 and mbr.mstride == mrp.extent*mrr.extent*mcr.extent and mpr.mstart == 0 and mpr.mstride == mrr.extent*mcr.extent and mrr.mstart == 0 and mrr.mstride == mcr.extent and mcr.mstart == 0 and mcr.mstride == 1");
+    if (msr.mstart != 0 || msr.mstride != mbr.mextent * mpr.mextent * mrr.mextent * mcr.mextent
+        || mbr.mstart != 0 || mbr.mstride != mpr.mextent * mrr.mextent * mcr.mextent
+        || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
+        || mrr.mstart != 0 || mrr.mstride != mcr.mextent
+        || mcr.mstart != 0 || mcr.mstride != 1)
+        throw std::runtime_error("A Tensor5View can only be converted to a plain C-array if it's pointing to a continuous block of data");
 
-  return mdata;
+    return mdata;
 }
 
 /** Conversion to plain C-array.
@@ -774,14 +777,14 @@ Numeric *Tensor5View::get_c_array()
 */
 const Numeric *Tensor5View::get_c_array() const
 {
-  if (msr.mstart != 0 || msr.mstride != mbr.mextent * mpr.mextent * mrr.mextent * mcr.mextent
-      || mbr.mstart != 0 || mbr.mstride != mpr.mextent * mrr.mextent * mcr.mextent
-      || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
-      || mrr.mstart != 0 || mrr.mstride != mcr.mextent
-      || mcr.mstart != 0 || mcr.mstride != 1)
-    throw runtime_error("A Tensor5View can only be converted to a plain C-array if mbr.mstart == 0 and mbr.mstride == mrp.extent*mrr.extent*mcr.extent and mpr.mstart == 0 and mpr.mstride == mrr.extent*mcr.extent and mrr.mstart == 0 and mrr.mstride == mcr.extent and mcr.mstart == 0 and mcr.mstride == 1");
+    if (msr.mstart != 0 || msr.mstride != mbr.mextent * mpr.mextent * mrr.mextent * mcr.mextent
+        || mbr.mstart != 0 || mbr.mstride != mpr.mextent * mrr.mextent * mcr.mextent
+        || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
+        || mrr.mstart != 0 || mrr.mstride != mcr.mextent
+        || mcr.mstart != 0 || mcr.mstride != 1)
+        throw std::runtime_error("A Tensor5View can only be converted to a plain C-array if it's pointing to a continuous block of data");
 
-  return mdata;
+    return mdata;
 }
 
 
@@ -882,7 +885,7 @@ ConstTensor5View::ConstTensor5View(Numeric *data,
 /** Output operator. This demonstrates how iterators can be used to
     traverse the tensor. We use the standard output operator for
     Tensor to print each book in turn. */
-ostream& operator<<(ostream& os, const ConstTensor5View& v)
+std::ostream& operator<<(std::ostream& os, const ConstTensor5View& v)
 {
   // Page iterators:
   ConstIterator5D is = v.begin();
@@ -2407,17 +2410,9 @@ Tensor5::Tensor5(const Tensor5& m) :
   \author Stefan Buehler
   \date   2002-12-19
 */
-Tensor5& Tensor5::operator=(const Tensor5& m)
+Tensor5& Tensor5::operator=(Tensor5 x)
 {
-  //  cout << "Tensor5 copy: m = " << m.nrows() << " " << m.ncols() << "\n";
-  //  cout << "              n = " << nrows() << " " << ncols() << "\n";
-
-  resize( m.msr.mextent,
-          m.mbr.mextent,
-          m.mpr.mextent,
-          m.mrr.mextent,
-          m.mcr.mextent );
-  copy( m.begin(), m.end(), begin() );
+  swap(*this, x);
   return *this;
 }
 
@@ -2471,6 +2466,19 @@ void Tensor5::resize(Index s, Index b, Index p, Index r, Index c)
     }
 }
 
+
+/** Swaps two objects. */
+void swap(Tensor5& t1, Tensor5& t2)
+{
+  std::swap(t1.msr, t2.msr);
+  std::swap(t1.mbr, t2.mbr);
+  std::swap(t1.mpr, t2.mpr);
+  std::swap(t1.mrr, t2.mrr);
+  std::swap(t1.mcr, t2.mcr);
+  std::swap(t1.mdata, t2.mdata);
+}
+
+
 /** Destructor for Tensor5. This is important, since Tensor5 uses new to
     allocate storage. */
 Tensor5::~Tensor5()
