@@ -1,4 +1,5 @@
-/* Copyright (C) 2001-2008 Stefan Buehler <sbuehler@ltu.se>
+
+/* Copyright (C) 2001-2012 Stefan Buehler <sbuehler@ltu.se>
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -38,10 +39,6 @@ public:
   /** Default constructor. */
   Iterator7D() : msv(), mstride(0) { /* Nothing to do here. */ }
 
-  /** Copy constructor. */
-  Iterator7D(const Iterator7D& o) : msv(o.msv), mstride(o.mstride)
-      { /* Nothing to do here. */ }
-
   /** Explicit constructor. */
   Iterator7D(const Tensor6View& x, Index stride) : msv(x), mstride(stride)
       { /* Nothing to do here. */ }
@@ -96,10 +93,6 @@ public:
   /** Default constructor. */
   ConstIterator7D() : msv(), mstride(0) { /* Nothing to do here. */ }
 
-  /** Copy constructor. */
-  ConstIterator7D(const ConstIterator7D& o) : msv(o.msv), mstride(o.mstride)
-    { /* Nothing to do here. */ }
-
   /** Explicit constructor. */
   ConstIterator7D(const ConstTensor6View& x, Index stride)
     : msv(x), mstride(stride)
@@ -713,6 +706,14 @@ public:
         CHECK(p);
         CHECK(r);
         CHECK(c);
+        return get(l, v, s, b, p, r, c);
+      }
+
+  /** Get element implementation without assertions. */
+  Numeric get( Index        l,
+               Index        v, Index        s, Index        b,
+               Index        p, Index        r, Index        c) const
+      {
         return                *(mdata + OFFSET(l) +
                                 OFFSET(v) + OFFSET(s) + OFFSET(b) +
                                 OFFSET(p) + OFFSET(r) + OFFSET(c)    );
@@ -1310,6 +1311,12 @@ public:
                        Index        p, Index        r, Index        c) const
       { return ConstTensor7View::operator()(l,v,s,b,p,r,c); }
 
+  /** Get element implementation without assertions. */
+  Numeric get( Index        l,
+                       Index        v, Index        s, Index        b,
+                       Index        p, Index        r, Index        c) const
+      { return ConstTensor7View::get(l,v,s,b,p,r,c); }
+
 
   // Non-const index operators:
 
@@ -1847,12 +1854,24 @@ public:
         CHECK(p);
         CHECK(r);
         CHECK(c);
+        return get(l, v, s, b, p, r, c);
+      }
+
+  /** Get element implementation without assertions. */
+  Numeric& get( Index        l,
+                Index        v, Index        s, Index        b,
+                Index        p, Index        r, Index        c)
+      {
         return                *(mdata + OFFSET(l) +
                                 OFFSET(v) + OFFSET(s) + OFFSET(b) +
                                 OFFSET(p) + OFFSET(r) + OFFSET(c)    );
       }
 
 
+  // Conversion to a plain C-array
+  const Numeric *get_c_array() const;
+  Numeric *get_c_array();
+
   // Functions returning const iterators:
   ConstIterator7D begin() const;
   ConstIterator7D end() const;
@@ -1924,7 +1943,7 @@ public:
   Tensor7(const Tensor7& v);
 
   // Assignment operators:
-  Tensor7& operator=(const Tensor7& x);
+  Tensor7& operator=(Tensor7 x);
   Tensor7& operator=(Numeric x);
 
   // Resize function:
@@ -1932,6 +1951,9 @@ public:
               Index        v, Index        s, Index        b,
               Index        p, Index        r, Index        c);
 
+  // Swap function:
+  friend void swap(Tensor7& t1, Tensor7& t2);
+
   // Destructor:
   virtual ~Tensor7();
 };
@@ -1956,7 +1978,7 @@ Numeric max(const ConstTensor7View& x);
 
 Numeric min(const ConstTensor7View& x);
 
-ostream& operator<<(ostream& os, const ConstTensor7View& v);
+std::ostream& operator<<(std::ostream& os, const ConstTensor7View& v);
 
 ////////////////////////////////
 // Helper function for debugging
