@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2002-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -17,7 +17,7 @@
 
 /*!
   \file   special_interp.cc
-  \author Patrick Eriksson <Patrick.Eriksson@rss.chalmers.se>
+  \author Patrick Eriksson <Patrick.Eriksson@chalmers.se>
   \date   2002-11-14   
   
   \brief  Interpolation routines for special purposes.
@@ -41,7 +41,7 @@
   surface-type variables, with names where _atmfield_ is replaced with
   _atmsurface_. 
 
-  Possible surface-type variables are *r_geoid*, *z_surface* and one page
+  Possible surface-type variables are *z_surface* and one page
   of *z_field*.
 
 
@@ -54,8 +54,8 @@
 
 
   - Various functions connected to the atmospheric fields:
-  
-  See below to find what functions that exist.
+
+  - Interpolation of Gridded fields of specialö types:
 */
 
 #include <cmath>
@@ -271,12 +271,12 @@ Numeric interp_atmfield_by_gp(
 
 //! interp_cloudfield_gp2itw
 /*!
-    Converts atmospheric grid positions to weights for interpolation of a
+    Converts atmospheric a grid position to weights for interpolation of a
     field defined ONLY inside the cloudbox.
 
     That is, as interp_atmfield_gp2itw, but for cloudbox only variables.
 
-    The input grid positions shall be with respect to total grids. If grid
+    The input grid position shall be with respect to total grids. If grid
     positions already refer to grid parts inside the cloudbox, you can use 
     interp_atmfield_gp2itw.
 
@@ -284,13 +284,14 @@ Numeric interp_atmfield_by_gp(
     cloudbox field, and can be used for later calls of e.g.
     interp_atmfield_by_itw
 
-    \param   itw                Output: Interpolation weights.
-    \param   gp_p_out           Output: Pressure cloudbox grid positions.
-    \param   gp_lat_out         Output: Latitude cloudbox grid positions.
-    \param   gp_lon_out         Output: Longitude cloudbox grid positions.
-    \param   gp_p_in            Pressure grid positions.
-    \param   gp_lat_in          Latitude grid positions.
-    \param   gp_lon_in          Longitude grid positions.
+    \param   itw                Output: Interpolation weights. Vector must be 
+                                given correct size before call of function.
+    \param   gp_p_out           Output: Pressure cloudbox grid position.
+    \param   gp_lat_out         Output: Latitude cloudbox grid positionn.
+    \param   gp_lon_out         Output: Longitude cloudbox grid position.
+    \param   gp_p_in            Pressure grid position.
+    \param   gp_lat_in          Latitude grid position.
+    \param   gp_lon_in          Longitude grid position.
     \param   atmosphere_dim     As the WSV with the same name.
     \param   cloudbiox_limits   As the WSV with the same name.
 
@@ -298,74 +299,48 @@ Numeric interp_atmfield_by_gp(
     \date   2010-02-12
 */
 void interp_cloudfield_gp2itw( 
-              Matrix&           itw, 
-              ArrayOfGridPos&   gp_p_out,
-              ArrayOfGridPos&   gp_lat_out,
-              ArrayOfGridPos&   gp_lon_out,
-        const ArrayOfGridPos&   gp_p_in,
-        const ArrayOfGridPos&   gp_lat_in,
-        const ArrayOfGridPos&   gp_lon_in,
-        const Index&            atmosphere_dim,
-        const ArrayOfIndex&     cloudbox_limits )
+              VectorView      itw, 
+              GridPos&        gp_p_out,
+              GridPos&        gp_lat_out,
+              GridPos&        gp_lon_out,
+        const GridPos&        gp_p_in,
+        const GridPos&        gp_lat_in,
+        const GridPos&        gp_lon_in,
+        const Index&          atmosphere_dim,
+        const ArrayOfIndex&   cloudbox_limits )
 {
-  const Index n = gp_p_in.nelem();
-
   // Shift grid positions to cloud box grids
   if( atmosphere_dim == 1 )
     {
-      gp_p_out.resize(n);
-      for (Index i = 0; i < n; i++ ) 
-        { 
-          gridpos_copy( gp_p_out[i], gp_p_in[i] ); 
-          gp_p_out[i].idx -= cloudbox_limits[0]; 
-        }
-      const Index n1 = cloudbox_limits[1] - cloudbox_limits[0];
-      gridpos_upperend_check( gp_p_out, n1 );
-      //
-      itw.resize(n,2);
+      gridpos_copy( gp_p_out, gp_p_in ); 
+      gp_p_out.idx -= cloudbox_limits[0]; 
+      gridpos_upperend_check( gp_p_out, cloudbox_limits[1]-cloudbox_limits[0] );
+      assert(itw.nelem() == 2 );
       interpweights( itw, gp_p_out );
     }
   else if( atmosphere_dim == 2 )
     {
-      gp_p_out.resize(n);
-      gp_lat_out.resize(n);
-      for (Index i = 0; i < n; i++ ) 
-        { 
-          gridpos_copy( gp_p_out[i], gp_p_in[i] ); 
-          gridpos_copy( gp_lat_out[i], gp_lat_in[i] ); 
-          gp_p_out[i].idx   -= cloudbox_limits[0]; 
-          gp_lat_out[i].idx -= cloudbox_limits[2]; 
-        }
-      const Index n1 = cloudbox_limits[1] - cloudbox_limits[0];
-      const Index n2 = cloudbox_limits[3] - cloudbox_limits[2];
-      gridpos_upperend_check( gp_p_out,   n1 );
-      gridpos_upperend_check( gp_lat_out, n2 );
-      //
-      itw.resize(n,4);
+      gridpos_copy( gp_p_out, gp_p_in ); 
+      gridpos_copy( gp_lat_out, gp_lat_in ); 
+      gp_p_out.idx   -= cloudbox_limits[0]; 
+      gp_lat_out.idx -= cloudbox_limits[2]; 
+      gridpos_upperend_check( gp_p_out,  cloudbox_limits[1]-cloudbox_limits[0]);
+      gridpos_upperend_check( gp_lat_out,cloudbox_limits[3]-cloudbox_limits[2]);
+      assert(itw.nelem() == 4 );
       interpweights( itw, gp_p_out, gp_lat_out );
     }
   else 
     {
-      gp_p_out.resize(n);
-      gp_lat_out.resize(n);
-      gp_lon_out.resize(n);
-      for (Index i = 0; i < n; i++ ) 
-        { 
-          gridpos_copy( gp_p_out[i], gp_p_in[i] ); 
-          gridpos_copy( gp_lat_out[i], gp_lat_in[i] ); 
-          gridpos_copy( gp_lon_out[i], gp_lon_in[i] ); 
-          gp_p_out[i].idx   -= cloudbox_limits[0]; 
-          gp_lat_out[i].idx -= cloudbox_limits[2]; 
-          gp_lon_out[i].idx -= cloudbox_limits[4];
-        }
-      const Index n1 = cloudbox_limits[1] - cloudbox_limits[0];
-      const Index n2 = cloudbox_limits[3] - cloudbox_limits[2];
-      const Index n3 = cloudbox_limits[5] - cloudbox_limits[4];
-      gridpos_upperend_check( gp_p_out,   n1 );
-      gridpos_upperend_check( gp_lat_out, n2 );
-      gridpos_upperend_check( gp_lon_out, n3 );
-      //
-      itw.resize(n,8);
+      gridpos_copy( gp_p_out, gp_p_in ); 
+      gridpos_copy( gp_lat_out, gp_lat_in ); 
+      gridpos_copy( gp_lon_out, gp_lon_in ); 
+      gp_p_out.idx   -= cloudbox_limits[0]; 
+      gp_lat_out.idx -= cloudbox_limits[2]; 
+      gp_lon_out.idx -= cloudbox_limits[4];
+      gridpos_upperend_check( gp_p_out,  cloudbox_limits[1]-cloudbox_limits[0]);
+      gridpos_upperend_check( gp_lat_out,cloudbox_limits[3]-cloudbox_limits[2]);
+      gridpos_upperend_check( gp_lon_out,cloudbox_limits[5]-cloudbox_limits[4]);
+      assert(itw.nelem() == 8 );
       interpweights( itw, gp_p_out, gp_lat_out, gp_lon_out );
     }
 }
@@ -555,183 +530,6 @@ Numeric interp_atmsurface_by_gp(
 
 
 
-//! interp_gfield3
-/*!
-    Single point interpolation of a GriddedField3.
-
-    Data where not all dimensions are effectively used can be
-    handled. For example, if the last dimension is not ysed,
-    *effective_dim* is set to 2.
-
-    "First dimension" refers to the left-most dimension. 
-
-    \param   value           Output: Obtained value.
-    \param   gfield3         Field to interpolate
-    \param   effective_dim   Effective dimension of gfield. See above.
-    \param   x               First dimension position of interpolation point.
-    \param   y               Second dimension position of interpolation point.
-    \param   z               Third dimension position of interpolation point.
-    \param   dim0            Expected name of grid for first dimension
-    \param   dim1            Expected name of grid for second dimension
-    \param   dim2            Expected name of grid for third dimension
-
-    \author Patrick Eriksson 
-    \date   2008-10-01
-*/
-void interp_gfield3( 
-                 Numeric&   value,
-           const GriddedField3&   gfield3,
-           const Index&     effective_dim,
-           const Numeric&   x,
-           const Numeric&   y,
-           const Numeric&   z,
-           const String&    dim0,
-           const String&    dim1,
-           const String&    dim2 )
-{
-  chk_if_in_range( "effective_dim", effective_dim, 1, 3 );
-
-  ArrayOfGridPos gp0(1), gp1(1), gp2(1);
-
-  // Check and grid position, dimension 0
-  //
-    {
-      const String gridname = gfield3.get_grid_name(0);
-      const ConstVectorView grid = gfield3.get_numeric_grid(0);
-      //
-      if( dim0 != gridname )
-        {
-          ostringstream os;
-          os << "Wrong quantity found for grid of first dimension:\n"
-             << "  expected quantity  : " << dim0 << "\n"
-             << "  quantity in gfield : " << gridname << "\n";
-          throw runtime_error( os.str() );
-          }
-      //
-      chk_if_increasing( "first grid of gfield3", grid );
-      //
-      if( x < grid[0]  ||  x > last(grid) )
-        {
-          ostringstream os;
-          os << "Interpolation outside covered range for first dimension:\n"
-             << "  interpolation point : " << x << "\n"
-             << "  gfield grid range   : [" << grid[0] << "," << last(grid) 
-             << "]\n";
-          throw runtime_error( os.str() );
-        }       
-      //
-      gridpos( gp0, grid, Vector(1,x) );
-    }
-
-  // Check and grid position, dimension 2
-  //
-  if( effective_dim >= 2 )
-    {
-      const String gridname = gfield3.get_grid_name(1);
-      const ConstVectorView grid = gfield3.get_numeric_grid(1);
-      //
-      if( dim1 != gridname )
-        {
-          ostringstream os;
-          os << "Wrong quantity found for grid of second dimension:\n"
-             << "  expected quantity  : " << dim1 << "\n"
-             << "  quantity in gfield : " << gridname << "\n";
-          throw runtime_error( os.str() );
-          }
-      //
-      chk_if_increasing( "second grid of gfield3", grid );
-      //
-      if( y < grid[0]  ||  y > last(grid) )
-        {
-          ostringstream os;
-          os << "Interpolation outside covered range for second dimension:\n"
-             << "  interpolation point : " << y << "\n"
-             << "  gfield grid range   : [" << grid[0] << "," << last(grid) 
-             << "]\n";
-          throw runtime_error( os.str() );
-        }       
-      //
-      gridpos( gp1, grid, Vector(1,y) );
-    }
-
-  // Check and grid position, dimension 2
-  //
-  if( effective_dim >= 3 )
-    {
-      const String gridname = gfield3.get_grid_name(2);
-      const ConstVectorView grid = gfield3.get_numeric_grid(2);
-      //
-      if( dim2 != gridname )
-        {
-          ostringstream os;
-          os << "Wrong quantity found for grid of third dimension:\n"
-             << "  expected quantity  : " << dim2 << "\n"
-             << "  quantity in gfield : " << gridname << "\n";
-          throw runtime_error( os.str() );
-          }
-      //
-      chk_if_increasing( "third grid of gfield3", grid );
-      //
-      if( z < grid[0]  ||  z > last(grid) )
-        {
-          ostringstream os;
-          os << "Interpolation outside covered range for second dimension:\n"
-             << "  interpolation point : " << z << "\n"
-             << "  gfield grid range   : [" << grid[0] << "," << last(grid) 
-             << "]\n";
-          throw runtime_error( os.str() );
-        }       
-      //
-      gridpos( gp2, grid, Vector(1,z) );
-    }
-
-  // Perform interpolation
-  //
-  Vector result( 1 );
-  //
-  if( effective_dim == 1 )
-    {
-      if( gfield3.data.nrows() > 1  ||  gfield3.data.ncols() > 1 )
-        {
-          ostringstream os;
-          os << "A 1D interpolation requested, but the provided gridded field "
-             << "has an effective dimension of 2D or 3D.";
-          throw runtime_error( os.str() );
-        }       
-        
-      Matrix itw(1,2);
-      interpweights( itw, gp0 );
-      interp( result, itw, gfield3.data(joker,0,0), gp0 );
-    }
-  //
-  else if( effective_dim == 2 )
-    {
-      if( gfield3.data.ncols() > 1 )
-        {
-          ostringstream os;
-          os << "A 2D interpolation requested, but the provided gridded field "
-             << "has an effective dimension of 3D.";
-          throw runtime_error( os.str() );
-        }       
-
-      Matrix itw(1,4);
-      interpweights( itw, gp0, gp1 );
-      interp( result, itw, gfield3.data(joker,joker,0), gp0, gp1 );
-    }
-  //
-  else if( effective_dim == 3 )
-    {
-      Matrix itw(1,8);
-      interpweights( itw, gp0, gp1, gp2 );
-      interp( result, itw, gfield3.data, gp0, gp1, gp2 );
-    }
-
-  value = result[0];
-}
-
-
-
-
 
 
 /*===========================================================================
@@ -822,6 +620,8 @@ void p2gridpos( ArrayOfGridPos& gp,
   gridpos( gp, logold, lognew, extpolfac );
 }
 
+
+
 //! p2gridpos_poly
 /*!
  Calculates grid positions for pressure values - higher order interpolation.
@@ -857,9 +657,97 @@ void p2gridpos_poly( ArrayOfGridPosPoly& gp,
 }
 
 
-/*===========================================================================
-  === Various functions connected to the atmospheric fields
-  ===========================================================================*/
+
+//! rte_pos2gridpos
+/*!
+   Converts a geographical position (rte_pos) to grid positions for p, 
+   lat and lon. 
+
+   The function calculates the altitude, latitude and longitude in *rte_pos* to
+   matching grid positions. The conversion is straightforwatd for latitude and
+   longitude. The altitude shall be converted pressure grid position which
+   requires an interpolation of z_field.
+
+   Handles 1D, 2D and 3D (gp_lat and gp_lon untouched if not used).
+
+   Note that the function performs several checks of runtime error type.
+
+   \param   gp_p        Output: Pressure grid position.
+   \param   gp_lat      Output: Latitude grid position.
+   \param   gp_lon      Output: Longitude grid position.
+   \param   atmosphere_dim  As the WSV with the same name.
+   \param   p_grid      As the WSV with the same name.
+   \param   lat_grid    As the WSV with the same name.
+   \param   lon_grid    As the WSV with the same name.
+   \param   z_field     As the WSV with the same name.
+   \param   rte_pos     As the WSV with the same name.
+
+   \author Patrick Eriksson
+   \date   2012-06-22
+*/
+void rte_pos2gridpos(
+         GridPos&     gp_p,
+         GridPos&     gp_lat,
+         GridPos&     gp_lon,
+   const Index&       atmosphere_dim,
+   ConstVectorView    p_grid,
+   ConstVectorView    lat_grid,
+   ConstVectorView    lon_grid,
+   ConstTensor3View   z_field,
+   ConstVectorView    rte_pos )
+{
+  chk_rte_pos( atmosphere_dim, rte_pos );
+
+  if( atmosphere_dim == 1 )
+    { 
+      chk_interpolation_grids( "Altitude interpolation", z_field(joker,0,0), 
+                                                                  rte_pos[0] );
+      gridpos( gp_p, z_field(joker,0,0), rte_pos[0] ); 
+    }
+  else
+    {
+      // Determine z at lat/lon (z_grid) by blue interpolation
+      const Index np = p_grid.nelem();
+      Vector z_grid( np );
+      ArrayOfGridPos agp_z, agp_lat(np);
+      //
+      gridpos_1to1( agp_z, p_grid );
+      //
+      chk_interpolation_grids( "Latitude interpolation", lat_grid, rte_pos[1] );
+      gridpos( gp_lat, lat_grid, rte_pos[1] );
+
+      if( atmosphere_dim == 2 )
+        {
+          for( Index i=0; i<np; i++ )
+            { agp_lat[i] = gp_lat; }
+          Matrix itw( np, 4 );
+          interpweights( itw, agp_z, agp_lat );
+          interp( z_grid, itw, z_field(joker,joker,0), agp_z, agp_lat );
+        }
+      else
+        {
+          chk_interpolation_grids( "Longitude interpolation", lon_grid, 
+                                                                  rte_pos[2] );
+          gridpos( gp_lon, lon_grid, rte_pos[2] );
+          ArrayOfGridPos agp_lon(np);
+          for( Index i=0; i<np; i++ )
+            { 
+              agp_lat[i] = gp_lat;  
+              agp_lon[i] = gp_lon; 
+            }
+          Matrix itw( np, 8 );
+          interpweights( itw, agp_z, agp_lat, agp_lon );
+          interp( z_grid, itw, z_field, agp_z, agp_lat, agp_lon );
+
+        }
+
+      // And use z_grid to get gp_p (gp_al and gp_lon determined above)
+      chk_interpolation_grids( "Altitude interpolation", z_grid, rte_pos[0] );
+      gridpos( gp_p, z_grid, rte_pos[0] );
+    }
+}
+
+
 
 //! z_at_lat_2d
 /*!
@@ -966,3 +854,119 @@ void z_at_latlon(
 
   z = z_tensor(Range(joker),0,0);
 }
+
+
+
+/*===========================================================================
+  === Interpolation of GriddedFields
+  ===========================================================================*/
+
+//! complex_n_interp
+/*!
+    General function for interpolating data of complex n type.
+
+    See documentation of comples_refr_index for format of complex_n-.
+
+    \param   n_real      Out: Real part [nf,nt]
+    \param   n_imag      Out: Imaginary part [nf,nt]
+    \param   complex_n   Complex refracton index data.
+    \param   varname     The name of complex_n to use in error message.
+    \param   f_grid      Output frequency grid [nf]
+    \param   t_grid      Output temperature grid [nt]
+
+    \author Patrick Eriksson 
+    \date   2013-08-16
+*/
+void complex_n_interp(
+         MatrixView       n_real,
+         MatrixView       n_imag,
+   const GriddedField3&   complex_n,
+   const String&          varname,
+   ConstVectorView        f_grid,
+   ConstVectorView        t_grid )
+{
+  // Set expected order of grids
+  Index gfield_fID    = 0;
+  Index gfield_tID    = 1;
+  Index gfield_compID = 2;
+
+  // Check of complex_n
+  //
+  complex_n.checksize_strict();
+  //
+  chk_griddedfield_gridname( complex_n, gfield_fID,    "Frequency"   );
+  chk_griddedfield_gridname( complex_n, gfield_tID,    "Temperature" );
+  chk_griddedfield_gridname( complex_n, gfield_compID, "Complex"     );
+  //
+  if( complex_n.data.ncols() != 2 )
+    {
+      ostringstream os;
+      os << "The data in *" << varname << "* must have exactly two pages. One page "
+         << "each\nfor the real and imaginary part of the complex refractive index.";
+    } 
+
+  // Frequency and temperature grid sizes
+  const Index nf_in = complex_n.data.npages();
+  const Index nt_in = complex_n.data.nrows(); 
+  const Index nf_out = f_grid.nelem();
+  const Index nt_out = t_grid.nelem();
+
+  //Assert size of output variables
+  assert( n_real.nrows() == nf_out  &&  n_real.ncols() == nt_out );
+  assert( n_imag.nrows() == nf_out  &&  n_imag.ncols() == nt_out );
+
+  const Vector& f_grid_in = complex_n.get_numeric_grid(gfield_fID);
+  const Vector& t_grid_in = complex_n.get_numeric_grid(gfield_tID);
+
+  // Expand/interpolate in frequency dimension
+  //
+  Matrix nrf(nf_out,nt_in), nif(nf_out,nt_in);
+  //
+  if( nf_in == 1 )
+    {
+      for( Index i=0; i<nf_out; i++ )
+        { 
+          nrf(i,joker) = complex_n.data(0,joker,0); 
+          nif(i,joker) = complex_n.data(0,joker,1); 
+        }
+    }
+  else
+    {
+      chk_interpolation_grids( "Frequency interpolation", f_grid_in, f_grid );
+      //
+      ArrayOfGridPos gp( nf_out );
+      Matrix         itw( nf_out, 2 );
+      gridpos( gp, f_grid_in, f_grid );
+      interpweights( itw, gp );
+      for( Index i=0; i<nt_in; i++ )
+        { 
+          interp( nrf(joker,i), itw, complex_n.data(joker,i,0), gp );
+          interp( nif(joker,i), itw, complex_n.data(joker,i,1), gp );
+        }
+    }
+
+  // Expand/interpolate in temperature dimension
+  //
+  if( nt_in == 1 )
+    {
+      for( Index i=0; i<nt_out; i++ )
+        { 
+          n_real(joker,i) = nrf(joker,0); 
+          n_imag(joker,i) = nif(joker,0); 
+        }
+    }
+  else
+    {
+      chk_interpolation_grids( "Temperature interpolation", t_grid_in, t_grid );
+      //
+      ArrayOfGridPos gp( nt_out );
+      Matrix         itw( nt_out, 2 );
+      gridpos( gp, t_grid_in, t_grid );
+      interpweights( itw, gp );
+      for( Index i=0; i<nf_out; i++ )
+        { 
+          interp( n_real(i,joker), itw, nrf(i,joker), gp );
+          interp( n_imag(i,joker), itw, nif(i,joker), gp );
+        }
+    }
+}
